@@ -35,6 +35,7 @@
 #include "panel-profile.h"
 #include "panel-gconf.h"
 #include "panel-util.h"
+#include "panel-globals.h"
 
 typedef struct {
 	PanelToplevel *toplevel;
@@ -75,6 +76,9 @@ typedef struct {
 
 	guint          toplevel_notify;
 	guint          background_notify;
+
+	/* The theme directory of the icon, see bug #119209 */
+	char          *icon_theme_dir;
 } PanelPropertiesDialog;
 
 static GQuark panel_properties_dialog_quark = 0;
@@ -98,6 +102,9 @@ panel_properties_dialog_free (PanelPropertiesDialog *dialog)
 		gtk_widget_destroy (dialog->properties_dialog);
 	dialog->properties_dialog = NULL;
 
+	g_free (dialog->icon_theme_dir);
+	dialog->icon_theme_dir = NULL;
+
 	g_free (dialog);
 }
 
@@ -288,8 +295,28 @@ static void
 panel_properties_dialog_icon_changed (PanelPropertiesDialog *dialog,
 				      GnomeIconEntry        *entry)
 {
-	panel_profile_set_attached_custom_icon (dialog->toplevel,
-						gnome_icon_entry_get_filename (entry));
+	const char *icon = NULL;
+        char       *freeme = NULL;
+                                                                                                                                                              
+        icon = gnome_icon_entry_get_filename (entry);
+                                                                                                                                                              
+        /* Strip dir from the icon path if in the icon
+         * theme directory.  See bug #119209
+         */
+        if (icon && g_path_is_absolute (icon)) {
+                char *dir;
+                                                                                                                                                              
+                dir = g_path_get_dirname (icon);
+                                                                                                                                                              
+                if (dir && dialog->icon_theme_dir && strcmp (dir, dialog->icon_theme_dir) == 0)
+                        icon = freeme = g_path_get_basename (icon);
+                                                                                                                                                              
+                g_free (dir);
+        }
+                                                                                                                                                              
+        panel_profile_set_attached_custom_icon (dialog->toplevel, icon);
+                                                                                                                                                              
+        g_free (freeme);
 }
 
 static void
@@ -307,10 +334,32 @@ panel_properties_dialog_setup_icon_entry (PanelPropertiesDialog *dialog,
 	dialog->icon_label = glade_xml_get_widget (gui, "icon_label");
 	g_return_if_fail (dialog->icon_label != NULL);
 
+	dialog->icon_theme_dir = NULL;
 	custom_icon = panel_profile_get_attached_custom_icon (dialog->toplevel);
+	if (custom_icon != NULL &&
+	    ! g_path_is_absolute (custom_icon)) {
+		/* if the icon is not absolute path name it comes from the
+		   theme, and as such we wish to store the theme directory
+		   where it comes from.  See bug #119209 */
+		char *icon = gnome_desktop_item_find_icon (panel_icon_theme,
+							   custom_icon,
+							   48 /* FIXME: preffered_size */,
+							   0 /* flags */);
+		if (icon != NULL)
+			dialog->icon_theme_dir = g_path_get_dirname (icon);
+		g_free (custom_icon);
+		custom_icon = icon;
+	}
 	gnome_icon_entry_set_filename (GNOME_ICON_ENTRY (dialog->icon_entry), custom_icon);
 	g_free (custom_icon);
 
+	if (dialog->icon_theme_dir == NULL) {
+		/* use the default pixmap directory as the standard icon_theme_dir,
+		 * since the standard directory is themed */
+		g_object_get (G_OBJECT (dialog->icon_entry), "pixmap_subdir",
+			      &(dialog->icon_theme_dir), NULL);
+	}
+
 	g_signal_connect_swapped (dialog->icon_entry, "changed",
 				  G_CALLBACK (panel_properties_dialog_icon_changed), dialog);
 
@@ -858,7 +907,7 @@ panel_properties_dialog_new (PanelToplevel *toplevel,
 {
 	PanelPropertiesDialog *dialog;
 
-	dialog = g_new (PanelPropertiesDialog, 1);
+	dialog = g_new0 (PanelPropertiesDialog, 1);
 
 	g_object_set_qdata_full (G_OBJECT (toplevel),
 				 panel_properties_dialog_quark,
